#!/usr/bin/env python
import sys
import boto3
import vmtools

vm_root_path = vmtools.vm_root_grabber()
sys.path.append(vm_root_path)
from local_settings import *

class Main():
    """Class to manipulate aws ec2 resources

    instance variables:
    self.aws_profile
    self.aws_region
    self.session
    self.client_ec2
    self.availability_zones_list
    """

    def __init__(self, aws_profile, aws_region):
        """set instance variables, set instance aws connections

        keyword arguments:
        :type aws_profile: string
        :param aws_profile: the profile to use from ~/.aws/credentials to connect to aws
        :type aws_region: string
        :param aws_region: the region to use for the aws connection object (all resources will be created in this region)
        """
        self.aws_profile = aws_profile
        self.aws_region = aws_region
        # aws session
        self.session = boto3.Session(profile_name=self.aws_profile)
        # ec2 aws client
        self.client_ec2 = self.session.client('ec2', region_name=self.aws_region)
        # aws ec2 object (mainly used for creating and modifying resources)
        self.ec2 = self.session.resource('ec2', region_name=self.aws_region)
        # create a list of available availability zones
        availability_zones_state_dict = self.client_ec2.describe_availability_zones()
        self.availability_zones_list = []
        for availability_zone_dict in availability_zones_state_dict['AvailabilityZones']:
            if availability_zone_dict['State'] == 'available':
                self.availability_zones_list.append(availability_zone_dict['ZoneName'])

    def create_object_type_variables(self, object_type):
        """Take aws object_type and return tuple of (object_type_singular, object_type_plural)
        keyword arguments:
        :type object_type: string
        :param object_type: the type of object it is (ex instance, vpc, subnet etc)
        """
        # create object type for boto3 (object_types should end in 's') and messaging
        if object_type.endswith('s'):
            object_type_plural = object_type
            object_type_singular = object_type[:-1]
        else:
            object_type_plural = '{}s'.format(object_type)
            object_type_singular = object_type
        return (object_type_singular, object_type_plural)

    def analysis_aws_object_lookup_result(self, result, object_type, name, fail_condition=None):
        """Take the result from a aws object lookup, object_type, name and fail_condition (optional), fail or return the aws object
        keyword arguments:
        :type result: list
        :param result: the result of an aws object lookup (ex ec2.instances.filter(Filters=filters)) 
        :type object_type: string
        :param object_type: the type of object it is (ex instance, vpc, subnet etc)
        :type name: string
        :param name: the name of the object (usually tag name)
        :type fail_condition: string
        :param fail_condition: the condition which will cause this function to error and exit the program. possible values are: exists (fail if the object exists), doesnt_exist (fail if the object doesn't exisit), or None (if the object exists, return object, if it doesn't return None)
        """
        object_type_singular, object_type_plural = self.create_object_type_variables(object_type=object_type)
        if result:
            if len(result) == 1:
                aws_object = result[0]
                if fail_condition == 'exists':
                    fail_on_exists_message = 'Failure: A(n) {} with tag name "{}" already exists (object listed below). Botohelper requires "Name tags" to be unique. Quitting...\n{}'.format(object_type_singular, name, result)
                    print(fail_on_exists_message)
                    sys.exit(1)
            else:
                too_many_objects_message = 'Failure: filtering {} by tag name "{}" resulted in more than one {} objects (listed below). Botohelper requires "Name tags" to be unique. Quitting...\n{}'.format(object_type_plural, name, object_type_singular, result)
                print(too_many_objects_message)
                sys.exit(1)
        else:
            if fail_condition == 'doesnt_exist':
                fail_on_doesnt_exists_message = 'Failure: Could not find a {} with tag name "{}". Quitting...\n'.format(object_type_singular, name)
                print(fail_on_doesnt_exists_message)
                sys.exit(1)
            aws_object = None
        return aws_object




#!/usr/bin/env python
import sys
import os
import boto3
import time
import vmtools
from awsretry import AWSRetry
import botohelper.route53 as route53
import botohelper.botohelpermain
import botocore.exceptions
import botocore.client

vm_root_path = vmtools.vm_root_grabber()
sys.path.append(vm_root_path)
from local_settings import *

class S3(botohelper.botohelpermain.Main):
    """Class to manipulate aws ec2 resources

    public methods:

    instance variables:
    self.aws_profile
    self.aws_region
    self.session
    self.client_ec2
    self.ec2
    self.availability_zones_list
    self.route53
    """

    def __init__(self, aws_profile, aws_region):
        """set instance variables, set instance aws connections

        keyword arguments:
        :type aws_profile: string
        :param aws_profile: the profile to use from ~/.aws/credentials to connect to aws
        :type aws_region: string
        :param aws_region: the region to use for the aws connection object (all resources will be created in this region)
        """
        super().__init__(aws_profile=aws_profile, aws_region=aws_region)
        self.client_s3 = self.session.client('s3', region_name=self.aws_region)
        self.resource_s3 = self.session.resource('s3', region_name=self.aws_region)


    def get_matching_s3_objects(self, bucket, prefix='', suffix=''):
        """
        Return a generator of matching objects from an S3 bucket.
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        :type prefix: string or tuple
        :param prefix: (Optional) Only fetch objects whose key starts with
        :type suffix: string or tuple
        :param suffix: (Optional) Only fetch objects whose keys end with
        """
        kwargs = {'Bucket': bucket}
    
        # If the prefix is a single string (not a tuple of strings), we can
        # do the filtering directly in the S3 API.
        if isinstance(prefix, str):
            kwargs['Prefix'] = prefix
    
        while True:
    
            # The S3 API response is a large blob of metadata.
            # 'Contents' contains information about the listed objects.
            resp = self.client_s3.list_objects_v2(**kwargs)
    
            try:
                contents = resp['Contents']
            except KeyError:
                return
    
            for obj in contents:
                key = obj['Key']
                if key.startswith(prefix) and key.endswith(suffix):
                    yield obj
    
            # The S3 API is paginated, returning up to 1000 keys at a time.
            # Pass the continuation token into the next response, until we
            # reach the final page (when this field is missing).
            try:
                kwargs['ContinuationToken'] = resp['NextContinuationToken']
            except KeyError:
                break
    
    
    def get_matching_s3_keys(self, bucket, prefix='', suffix=''):
        """
        Generate the keys in an S3 bucket.
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        :type prefix: string or tuple
        :param prefix: (Optional) Only fetch objects whose key starts with
        :type suffix: string or tuple
        :param suffix: (Optional) Only fetch objects whose keys end with
        """
        for obj in self.get_matching_s3_objects(bucket, prefix, suffix):
            yield obj['Key']

    def is_s3_bucket_encrypted(self, bucket):
        """
        Check if the S3 bucket is has encryption set, if so return the 'Rules' list, if not return None
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        """
        try:
            response = self.client_s3.get_bucket_encryption(Bucket=bucket)
        except botocore.exceptions.ClientError as e:
            return None
        return response['ServerSideEncryptionConfiguration']['Rules']

    def is_s3_object_encrypted(self, bucket, key):
        """
        Check if the S3 object is encrypted, if so return True, if not return False
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        :type key: string
        :param key: The S3 key
        """
        s3_object = self.resource_s3.Object(bucket, key)
        if s3_object.server_side_encryption:
            return True
        else:
            return False

    def does_s3_bucket_exist(self, bucket):
        """
        Check if the S3 bucket exists, if so return True, otherwise return False
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        """
        try:
            self.resource_s3.meta.client.head_bucket(Bucket=bucket)
        except botocore.client.ClientError as e:
            return False
        return True

    def enable_default_encryption_on_s3_bucket(self, bucket):
        """
        Enable the default, transparent encryption (AES256) on the given bucket
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        """
        self.client_s3.put_bucket_encryption(Bucket=bucket, ServerSideEncryptionConfiguration={'Rules': [{'ApplyServerSideEncryptionByDefault': {'SSEAlgorithm': 'AES256'}}]})

    def disable_default_encryption_on_s3_bucket(self, bucket):
        """
        Disable encryption on the given bucket
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        """
        self.client_s3.delete_bucket_encryption(Bucket=bucket)

    def get_list_of_all_buckets(self):
        """
        Return a list of the all the buckets for the current account
        """
        all_bucket_objects_dict = self.client_s3.list_buckets()
        all_buckets_list = [ bucket_dict['Name'] for bucket_dict in all_bucket_objects_dict['Buckets'] ]
        return all_buckets_list

    def delete_bucket(self, bucket):
        """
        Delete bucket and all key within it
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        """
        bucket_object = self.resource_s3.Bucket(bucket) 
        for bucket_key in bucket_object.objects.all():
            bucket_key.delete()
        bucket_object.delete()

    def copy_s3_object_between_buckets(self, src_bucket, s3_object_key, dest_bucket, verbose=False):
        """
        Copy a single S3 object from one S3 bucket to another
        :type src_bucket: string
        :param src_bucket: Name of the source S3 bucket.
        :type s3_object_key: string
        :param s3_object_key: The key of the object to be copied
        :type dest_bucket: string
        :param dest_bucket: Name of the destination S3 bucket.
        :type verbose: boolean
        :param verbose: if True prints out the source and destination for each object copied
        """
        if verbose:
            copy_message = 'copying s3://{}/{} to s3://{}/{}'.format(src_bucket, s3_object_key, dest_bucket, s3_object_key)
            print(copy_message)
        response = self.client_s3.copy_object(Bucket=dest_bucket, Key=s3_object_key, CopySource={'Bucket': src_bucket, 'Key': s3_object_key})

    def create_s3_bucket(self, bucket_name, bucket_tags=False):
        """
        Copy all keys from src_bucket to dest_bucket
        :type bucket_name: string
        :param bucket_name: Name of the S3 bucket.
        :type bucket_tags: list of dicts
        :param bucket_tags: a list of tag dicts (if False no tags are created) ex: [{'Key': 'Name','Value': 'mybucket'}, {'Key': 'environment','Value': 'prod'}] 
        """
        bucket_object = self.resource_s3.create_bucket(Bucket=bucket_name)
        if bucket_tags:
            self.client_s3.put_bucket_tagging(Bucket=bucket_name,Tagging={'TagSet': bucket_tags } )

    def create_copy_of_s3_bucket(self, src_bucket, dest_bucket, copy_only_mising_objects=False, copy_only_unencrypted_objects=False, dest_bucket_tags=False, verbose=False):
        """
        Copy all keys from src_bucket to dest_bucket
    
        :type src_bucket: string
        :param src_bucket: Name of the source S3 bucket.
        :type dest_bucket: string
        :param dest_bucket: Name of the destination S3 bucket.
        :type copy_only_mising_objects: boolean
        :param copy_only_mising_objects: If True only the files not already in dest_bucket will be copied
        :type copy_only_unencrypted_objects: boolean
        :param copy_only_unencrypted_objects: If True only the unencrypted files src_bucket will be copied
        :type dest_bucket_tags: list of dicts
        :param dest_bucket_tags: a list of tag dicts (if False no tags are created) ex: [{'Key': 'Name','Value': 'mybucket'}, {'Key': 'environment','Value': 'prod'}] 
        :type verbose: boolean
        :param verbose: if True prints out the source and destination for each object copied
        """
        # make sure dest_bucket exists
        dest_bucket_object = self.resource_s3.create_bucket(Bucket=dest_bucket)
        if dest_bucket_tags:
            self.client_s3.put_bucket_tagging(Bucket=dest_bucket,Tagging={'TagSet': dest_bucket_tags } )
        src_bucket_object = self.resource_s3.Bucket(src_bucket)
        # create objects_to_copy_list based on missing objects if requested
        if copy_only_mising_objects:
            src_bucket_generator = self.get_matching_s3_keys(bucket=src_bucket)
            dest_bucket_generator = self.get_matching_s3_keys(bucket=dest_bucket)
            src_bucket_generator_set = set(src_bucket_generator)
            dest_bucket_generator_set = set(dest_bucket_generator)
            objects_to_copy_list = list(src_bucket_generator_set - dest_bucket_generator_set)
        # otherwise create objects_to_copy_list based on all files
        else:
            objects_to_copy_list = [ s3_object.key for s3_object in src_bucket_object.objects.all() ]
        # filter out the already encrypted if requested
        if copy_only_unencrypted_objects:
            updated_objects_to_copy_list = []
            for s3_key in objects_to_copy_list:
                if not self.is_s3_object_encrypted(bucket=src_bucket, key=s3_key):
                    updated_objects_to_copy_list.append(s3_key)
            objects_to_copy_list = updated_objects_to_copy_list
        # copy objects
        for s3_object_key in objects_to_copy_list:
            if verbose:
                copy_message = 'copying s3://{}/{} to s3://{}/{}'.format(src_bucket, s3_object_key, dest_bucket, s3_object_key)
                print(copy_message)
            response = self.client_s3.copy_object(Bucket=dest_bucket, Key=s3_object_key, CopySource={'Bucket': src_bucket, 'Key': s3_object_key})

    def download_s3_file(self, bucket, remote_file_to_download, local_absolute_path):
        """
        download the remote_file_to_download from s3 to local_absolute_path
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        :type remote_file_to_download: string
        :param remote_file_to_download: the key of the remote file to download from s3
        :type local_absolute_path: string
        :param local_absolute_path: the absolute path where to download remote_file_to_download
        """
        self.client_s3.download_file(bucket, remote_file_to_download, local_absolute_path)

    def request_restore_from_glacier(self, bucket, key, number_of_days):
        """
        Request a resotre from glaicer to s3 return the response
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        :type key: string
        :param key: the key of the remote file to restore from glacier
        :type number_of_days: int
        :param number_of_days: the number of days the restore should be available for (lowest would be 1 but a higher number may incur more cost)
        """
        bucket_obj = self.resource_s3.Bucket(bucket)
        object_summary_iterator = bucket_obj.objects.filter(Prefix=key)
        object_summary_list = []
        for object_summary in object_summary_iterator:
            object_summary_list.append(object_summary)
        if len(object_summary_list) == 0:
            return None
        elif len(object_summary_list) > 1:
            raise ValueError('object_summary_list should only have 1 or 0 object, key filter must not be specific enough')
        request_response = bucket_obj.meta.client.restore_object(Bucket=bucket, Key=key, RestoreRequest={'Days': number_of_days})
        return request_response

    def get_restore_expiration_date(self, bucket, key):
        """
        Return a dict with the expiration info of a glacier restore key, Return None if not restored
    
        :type bucket: string
        :param bucket: Name of the S3 bucket.
        :type key: string
        :param key: the key of the remote file
        """

        s3_object = self.resource_s3.Object(bucket,key)
        s3_object_expiration_dict = {}
        if s3_object.restore:
            for value_key_pair in s3_object.restore.split('", '):
                s3_object_expiration_dict[value_key_pair.split('="')[0]] = value_key_pair.split('="')[1].strip('"')
        else:
            return None

        return s3_object_expiration_dict





#!/usr/bin/env python
import sys
import os
import datetime
import boto3
import time
import vmtools
import botohelper.botohelpermain

vm_root_path = vmtools.vm_root_grabber()
sys.path.append(vm_root_path)
from local_settings import *

class Iam(botohelper.botohelpermain.Main):
    """Class to manipulate aws iam resources

    public methods:

    instance variables:
    self.aws_profile
    self.aws_region
    self.session
    self.iam
    """

    def __init__(self, aws_profile, aws_region):
        """set instance variables, set instance aws connections

        keyword arguments:
        :type aws_profile: string
        :param aws_profile: the profile to use from ~/.aws/credentials to connect to aws
        :type aws_region: string
        :param aws_region: the region to use for the aws connection object (all resources will be created in this region)
        """
        super().__init__(aws_profile=aws_profile, aws_region=aws_region)
        # aws iam object (mainly used for creating and modifying iam user, groups, etc)
        self.iam = self.session.resource('iam', region_name=self.aws_region)

    def get_object_from_name(self, tag_name, object_type, pathprefix=None, fail_condition=None):
        """Take tag_name and object_type then return aws_object or None depending on the fail_condition
        Note: this function will always fail if it finds more than one object with the same name tag
        keyword arguments:
        :type tag_name: string
        :param tag_name: the value of the Name tag for the aws_object
        :type object_type: string
        :param object_type: the type of iam object (options are: groups, instance_profiles, policies, roles, server_certificates, users, virtual_mfa_devices, saml_providers)
        :type pathprefix: string
        :param pathprefix: the pathprefix to use for filtering for example '/saltmaster_group_path/' (Note this is ignored for saml_providers and virtual_mfa_devices)
        :type fail_condition: string
        :param fail_condition: the condition which will cause this function to error and exit the program. possible values are: exists (fail if the object exists), doesnt_exist (fail if the object doesn't exisit), or None (if the object exists, return object, if it doesn't return None)
        """
        # get the object_type variables
        object_type_singular, object_type_plural = super().create_object_type_variables(object_type=object_type)
        # built 
        filters_dict = { 'PathPrefix': pathprefix }
        # filter all object of that type by our tag name and other stuff
        result = list(getattr(self.iam, object_type_plural).filter(**filters_dict))

        # check the result
        aws_object = super().analysis_aws_object_lookup_result(result=result, object_type=object_type, name=tag_name, fail_condition=fail_condition)
        return aws_object

#    def get_object_from_name(self, tag_name, object_type, aws_service, fail_condition=None):
#        list(iam.groups.filter(**filter_dict))
#        super().get_object_from_name(tag_name, object_type, aws_service, fail_condition=None)
#    def get_iam_object_from_name(self, tag_name, object_type, fail_condition=None):
#        """Take tag_name and object_type then return aws_object or None depending on the fail_condition
#        Note: this function will always fail if it finds more than one object with the same name tag
#        keyword arguments:
#        :type tag_name: string
#        :param tag_name: the value of the Name tag for the aws_object
#        :type object_type: string
#        :param object_type: the type of object it is (ex instance, vpc, subnet etc)
#        :type fail_condition: string
#        :param fail_condition: the condition which will cause this function to error and exit the program. possible values are: exists (fail if the object exists), doesnt_exist (fail if the object doesn't exisit), or None (if the object exists, return object, if it doesn't return None)
#        """
#        # create object type for boto3 (object_types should end in 's') and messaging
#        if object_type.endswith('s'):
#            object_type_plural = object_type
#            object_type_singular = object_type[:-1]
#        else:
#            object_type_plural = '{}s'.format(object_type)
#            object_type_singular = object_type
#        filters = [{'Name':'tag:Name', 'Values':[tag_name]}]
#        # filter all object of that type by our tag name and other stuff
#        result = list(getattr(self.iam, object_type_plural).filter(Filters=filters))
#        if result:
#            if len(result) == 1:
#                aws_object = result[0]
#                if fail_condition == 'exists':
#                    fail_on_exists_message = 'Failure: A(n) {} with tag name "{}" already exists (object listed below). Botohelper requires "Name tags" to be unique. Quitting...\n{}'.format(object_type_singular, tag_name, result)
#                    print(fail_on_exists_message)
#                    sys.exit(1)
#            else:
#                too_many_objects_message = 'Failure: filtering {} by tag name "{}" resulted in more than one {} objects (listed below). Botohelper requires "Name tags" to be unique. Quitting...\n{}'.format(object_type_plural, tag_name, object_type_singular, result)
#                print(too_many_objects_message)
#                sys.exit(1)
#        else:
#            if fail_condition == 'doesnt_exist':
#                fail_on_doesnt_exists_message = 'Failure: Could not find a {} with tag name "{}". Quitting...\n'.format(object_type_singular, tag_name)
#                print(fail_on_exists_message)
#                sys.exit(1)
#            aws_object = None
#        return aws_object

    def get_group_from_name(self, group_name):
        """Take group name if it exists return group object, if not return None
        keyword arguments:
        :type group_name: string
        :param group_name: the Name tag of the iam group
        """
        #doing the search with pure python is easier to follow instead of using boto3 filters
        list_of_groups = list(self.iam.groups.all())
        for group in list_of_groups:
            if group.name == group_name:
                return group
        return None



#!/usr/bin/env python
import sys
import os
import boto3
import time
import re
import vmtools
import botohelper.botohelpermain

vm_root_path = vmtools.vm_root_grabber()
sys.path.append(vm_root_path)
from local_settings import *

class Utility(botohelper.botohelpermain.Main):
    """Class to manipulate aws ec2 resources

    public methods:
    delete_vpc
    delete_instances 
    create_vpc
    create_subnet
    create_internet_gateway
    get_object_from_name
    get_environment_from_vpc_name
    get_default_route_table_object
    get_default_security_group_object 
    create_tag
    create_instance
    get_latest_ami_from_prefix

    instance variables:
    self.aws_profile
    self.aws_region
    self.session
    self.client_ec2
    self.ec2
    self.availability_zones_list
    self.route53
    """

    def __init__(self, aws_profile, aws_region):
        """set instance variables, set instance aws connections

        keyword arguments:
        :type aws_profile: string
        :param aws_profile: the profile to use from ~/.aws/credentials to connect to aws
        :type aws_region: string
        :param aws_region: the region to use for the aws connection object (all resources will be created in this region)
        """
        super().__init__(aws_profile=aws_profile, aws_region=aws_region)

    def build_collection_method_from_camel_method(self, camel_case_method):
        # exceptions_dict format: if collection_method_singular ends with 'key' add on the 'value' else add an 's'
        exceptions_dict = {
                'ss': 'es',
                'options': '_sets'
                }
        collection_method_singular = re.sub( '(?<!^)(?=[A-Z])', '_', camel_case_method ).lower()
        for ending in exceptions_dict:
            if collection_method_singular.endswith(ending):
                collection_method_plural = '{}{}'.format(collection_method_singular, exceptions_dict[ending])
                return collection_method_plural
        collection_method_plural = '{}s'.format(collection_method_singular)
        return collection_method_plural

    def build_resource_dict(self):
        """Create a vpc return vpc_id
        keyword arguments:
        :type vpc_name: string
        :param vpc_name: the Name tag for the vpc
        :type cidr_block: string
        :param cidr_block: cidr_block for the new vpc (ex '10.0.1.0/24')
        :type environment: string
        :param environment: the enviroment tag for the vpc
        """
        known_missiong_collection_methods_list = ['network_interface_associations', 'routes', 'route_table_associations', 'tags']
        ec2_resource_list = dir(self.ec2)
        camel_case_methods_list = [ method for method in ec2_resource_list if method[0].isupper() ]
        camel_case_method_dict = {}
        for camel_case_method in camel_case_methods_list:
            collection_method = self.build_collection_method_from_camel_method(camel_case_method=camel_case_method)
            if collection_method in ec2_resource_list:
                camel_case_method_dict[camel_case_method] =  collection_method
            elif collection_method in known_missiong_collection_methods_list:
                camel_case_method_dict[camel_case_method] =  None
            else:
                collection_method_not_found_message = 'WARNING: {} not found in ec2_resource_list for camel_case_method: {}'.format(collection_method, camel_case_method)
                print(collection_method_not_found_message)
        print(camel_case_method_dict)
        return camel_case_method_dict

    def get_state_method_for_resource(self, resource_type):
        print(resource_type)
        if resource_type == 'images':
            sample_resource_list = list(self.ec2.images.filter(Owners=['self']))
        else:
            sample_resource_list = list(getattr(self.ec2, resource_type).all())
        if sample_resource_list:
            sample_resource = sample_resource_list[0]
            print(dir(sample_resource))

    def repopulate_state_dict(self):
        camel_case_method_dict = self.build_resource_dict()
        for camel_case_method, collection_method in camel_case_method_dict.items():
            if collection_method:
                self.get_state_method_for_resource(resource_type=camel_case_method_dict[camel_case_method])

aws_object = Utility(aws_profile='dev', aws_region='us-east-1')
aws_object.repopulate_state_dict()

